@@ -21,13 +21,12 @@
 """Read and write DBnomics JSON and TSV files from the filesystem."""
 
 
+import json
 import logging
 from pathlib import Path
 
-import orjson as json
-
-from . import files
 from ..exceptions import StorageError
+from . import files
 from .abstract import AbstractDatasetDir, AbstractStorage
 
 log = logging.getLogger(__name__)
@@ -186,13 +185,13 @@ class FileSystemDatasetDir(AbstractDatasetDir):
 
 def load_json_file(storage, path):
     try:
-        text = path.read_text()
+        with path.open() as fd:
+            try:
+                return json.load(fd)
+            except ValueError:
+                return None
     except FileNotFoundError:
         return None
-    try:
-        return json.loads(text)
-    except ValueError:
-        return None
 
 
 def load_text_file(storage, path):
