@@ -30,16 +30,15 @@ https://www.dulwich.io/docs/tutorial/file-format.html
 """
 
 
+import json
 import logging
 from io import StringIO
 from pathlib import Path
 
 from dulwich.objects import Blob
 
-import orjson as json
-
-from . import files
 from ..exceptions import StorageError
+from . import files
 from .abstract import AbstractDatasetDir, AbstractStorage
 
 git_blob_filemode = 0o100644
