@@ -21,12 +21,11 @@
 """JSON schema functions"""
 
 
+import json
 import pkgutil
 
 import jsonschema
 
-import orjson as json
-
 
 def load_schema(json_file_name):
     json_schema_str = pkgutil.get_data("dbnomics_data_model", "schemas/{}".format(json_file_name)).decode('utf-8')
